// Copyright (c) 2015, Devon Carew. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

/**
 * A Dart library to access the Librato hosted statistics service
 * (https://metrics.librato.com).
 */
library librato;

import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:crypto/crypto.dart';

/**
 * A class used to send statistical data to the Librato service
 * (https://metrics.librato.com).
 *
 * To use this class, instantiate an instance (with an optional url), and call
 * [postStats] to post a set of statistics.
 */
class Librato {
  static const String DEFAULT_URL = "https://metrics-api.librato.com/v1/metrics";

  static String _env(String key) {
    if (Platform.environment[key] == null) throw 'Missing env var: ${key}';
    return Platform.environment[key];
  }

  final String url;
  final String _username;
  final String _password;

  Librato(this._username, this._password, [this.url = DEFAULT_URL]);

  /**
   * Create a new `Librato` instance. This requires the `LIBRATO_USER` and
   * `LIBRATO_TOKEN` environment variables to be set (and will throw otherwise).
   */
  Librato.fromEnvVars() : url = DEFAULT_URL,
      _username = _env('LIBRATO_USER'), _password = _env('LIBRATO_TOKEN');

  /**
   * Send a group of statistics. The `groupName` paremeter corresponds to the
   * Librato `source`. The keys of the `stats` map should be the names of the
   * statistic, and the cooresponding value in the map should be the numeric
   * value of the statistic.
   */
  Future postStats(Map<String, num> stats, {String groupName}) {
    var statsList = stats.keys.map((key) {
      Map m = { 'name': key, 'value': stats[key] };
      if (groupName != null) m['source'] = groupName;
      return m;
    }).toList();

    //  { "gauges": [ {...}, {...} ] }
    Map m = { 'gauges': statsList };
    String data = JSON.encode(m);

    HttpClient client = new HttpClient();
    return client.postUrl(Uri.parse(url)).then((HttpClientRequest request) {
      final auth = CryptoUtils.bytesToBase64(
          UTF8.encode("${_username}:${_password}"));
      request.headers.set('authorization', "Basic ${auth}");
      request.headers.contentType = ContentType.JSON;
      request.write(data);
      return request.close();
    }).then((HttpClientResponse response) {
      return response.drain();
    });
  }
}
